@@ -42,16 +42,15 @@ The following functions return the names of various files of interest:
 """
 
 
-import                        collections
-import                        logging
-import os.path             as op
-import numpy               as np
+import                   collections
+import                   logging
+import os.path        as op
+import numpy          as np
 
-import fsl.utils.path      as fslpath
-import fsl.utils.transform as transform
-
-from . import image as fslimage
-from . import          featdesign
+import fsl.utils.path       as fslpath
+import fsl.transform.affine as affine
+from . import image         as fslimage
+from . import                  featdesign
 
 
 log = logging.getLogger(__name__)
@@ -64,7 +63,7 @@ def isFEATImage(path):
 
     try:
         path = fslimage.addExt(path, mustExist=True)
-    except:
+    except fslimage.PathError:
         return False
 
     dirname  = op.dirname( path)
@@ -115,7 +114,7 @@ def hasStats(featdir):
     try:
         getZStatFile(featdir, 0)
         return True
-    except:
+    except fslimage.PathError:
         return False
 
 
@@ -357,6 +356,9 @@ def loadClusterResults(featdir, settings, contrast):
         clusterFile = op.join(
             featdir, 'cluster_zstat{}_std.txt'.format(contrast + 1))
 
+        if not op.exists(clusterFile):
+            return None
+
         # In higher levle analysis run in some standard
         # space, the cluster coordinates are in standard
         # space. We transform them to voxel coordinates.
@@ -365,9 +367,6 @@ def loadClusterResults(featdir, settings, contrast):
             getDataFile(featdir),
             loadData=False).worldToVoxMat
 
-        if not op.exists(clusterFile):
-            return None
-
     log.debug('Loading cluster results for contrast {} from {}'.format(
         contrast, clusterFile))
 
@@ -385,6 +384,12 @@ def loadClusterResults(featdir, settings, contrast):
 
                 setattr(self, attrName, val)
 
+            # if cluster thresholding was not used,
+            # the cluster table will not contain
+            # P valuse.
+            if not hasattr(self, 'p'):    self.p    = 1.0
+            if not hasattr(self, 'logp'): self.logp = 0.0
+
     # This dict provides a mapping between
     # Cluster object attribute names, and
     # the corresponding column name in the
@@ -427,48 +432,49 @@ def loadClusterResults(featdir, settings, contrast):
         # whitespace, and discarding
         # empty lines
         lines = f.readlines()
-        lines = [l.strip() for l in lines]
-        lines = [l         for l in lines if l != '']
 
-        # the first line should contain column
-        # names, and each other line should
-        # contain the data for one cluster
-        colNames     = lines[0]
-        clusterLines = lines[1:]
+    lines = [line.strip() for line in lines]
+    lines = [line         for line in lines if line != '']
 
-        # each line should be tab-separated
-        colNames     =  colNames.split('\t')
-        clusterLines = [cl      .split('\t') for cl in clusterLines]
+    # the first line should contain column
+    # names, and each other line should
+    # contain the data for one cluster
+    colNames     = lines[0]
+    clusterLines = lines[1:]
 
-        # Turn each cluster line into a
-        # Cluster instance. An error will
-        # be raised if the columm names
-        # are unrecognised (i.e. not in
-        # the colmap above), or if the
-        # file is poorly formed.
-        clusters = [Cluster(**dict(zip(colNames, cl))) for cl in clusterLines]
+    # each line should be tab-separated
+    colNames     =  colNames.split('\t')
+    clusterLines = [cl      .split('\t') for cl in clusterLines]
 
-        # Make sure all coordinates are in voxels -
-        # for first level analyses, the coordXform
-        # will be an identity transform (the coords
-        # are already in voxels). But for higher
-        # level, the coords are in mm, and need to
-        # be transformed to voxels.
-        for c in clusters:
+    # Turn each cluster line into a
+    # Cluster instance. An error will
+    # be raised if the columm names
+    # are unrecognised (i.e. not in
+    # the colmap above), or if the
+    # file is poorly formed.
+    clusters = [Cluster(**dict(zip(colNames, cl))) for cl in clusterLines]
 
-            zmax    = [c.zmaxx,    c.zmaxy,    c.zmaxz]
-            zcog    = [c.zcogx,    c.zcogy,    c.zcogz]
-            copemax = [c.copemaxx, c.copemaxy, c.copemaxz]
+    # Make sure all coordinates are in voxels -
+    # for first level analyses, the coordXform
+    # will be an identity transform (the coords
+    # are already in voxels). But for higher
+    # level, the coords are in mm, and need to
+    # be transformed to voxels.
+    for c in clusters:
 
-            zmax    = transform.transform([zmax],    coordXform)[0].round()
-            zcog    = transform.transform([zcog],    coordXform)[0].round()
-            copemax = transform.transform([copemax], coordXform)[0].round()
+        zmax    = [c.zmaxx,    c.zmaxy,    c.zmaxz]
+        zcog    = [c.zcogx,    c.zcogy,    c.zcogz]
+        copemax = [c.copemaxx, c.copemaxy, c.copemaxz]
 
-            c.zmaxx,   c.zmaxy,    c.zmaxz    = zmax
-            c.zcogx,   c.zcogy,    c.zcogz    = zcog
-            c.copemax, c.copemaxy, c.copemaxz = copemax
+        zmax    = affine.transform([zmax],    coordXform)[0].round()
+        zcog    = affine.transform([zcog],    coordXform)[0].round()
+        copemax = affine.transform([copemax], coordXform)[0].round()
 
-        return clusters
+        c.zmaxx,   c.zmaxy,    c.zmaxz    = zmax
+        c.zcogx,   c.zcogy,    c.zcogz    = zcog
+        c.copemax, c.copemaxy, c.copemaxz = copemax
+
+    return clusters
 
 
 def getDataFile(featdir):
