@@ -13,6 +13,8 @@ paths.
 
    deepest
    shallowest
+   allFiles
+   hasExt
    addExt
    removeExt
    getExt
@@ -20,11 +22,14 @@ paths.
    getFileGroup
    removeDuplicates
    uniquePrefix
+   commonBase
 """
 
 
-import            glob
 import os.path as op
+import            os
+import            glob
+import            operator
 
 
 class PathError(Exception):
@@ -62,7 +67,7 @@ def shallowest(path, suffixes):
     path = path.strip()
 
     # We've reached the root of the file system
-    if path == op.sep or path == '':
+    if path == op.sep or path == '' or op.splitdrive(path)[1] == '':
         return None
 
     path   = path.rstrip(op.sep)
@@ -77,11 +82,33 @@ def shallowest(path, suffixes):
     return None
 
 
+def allFiles(root):
+    """Return a list containing all files which exist underneath the specified
+    ``root`` directory.
+    """
+
+    files = []
+
+    for dirpath, _, filenames in os.walk(root):
+        filenames = [op.join(dirpath, f) for f in filenames]
+        files.extend(filenames)
+
+    return files
+
+
+def hasExt(path, allowedExts):
+    """Convenience function which returns ``True`` if the given ``path``
+    ends with any of the given ``allowedExts``, ``False`` otherwise.
+    """
+    return any([path.endswith(e) for e in allowedExts])
+
+
 def addExt(prefix,
            allowedExts=None,
            mustExist=True,
            defaultExt=None,
-           fileGroups=None):
+           fileGroups=None,
+           unambiguous=True):
     """Adds a file extension to the given file ``prefix``.
 
     If ``mustExist`` is False, and the file does not already have a
@@ -95,8 +122,8 @@ def addExt(prefix,
 
        - No files exist with the given prefix and a supported extension.
 
-       - ``fileGroups`` is ``None``, and more than one file exists with the
-         given prefix, and a supported extension.
+       - ``fileGroups is None`` and ``unambiguous is True``, and more than
+         one file exists with the given prefix, and a supported extension.
 
     Otherwise the full file name is returned.
 
@@ -109,6 +136,11 @@ def addExt(prefix,
     :arg defaultExt:  Default file extension to use.
 
     :arg fileGroups:  Recognised file groups - see :func:`getFileGroup`.
+
+    :arg unambiguous: If ``True`` (the default), and more than one file
+                      exists with the specified ``prefix``, a
+                      :exc:`PathError` is raised. Otherwise, a list
+                      containing *all* matching files is returned.
     """
 
     if allowedExts is None: allowedExts = []
@@ -121,7 +153,7 @@ def addExt(prefix,
 
         # the provided file name already
         # ends with a supported extension
-        if any([prefix.endswith(ext) for ext in allowedExts]):
+        if hasExt(prefix, allowedExts):
             return prefix
 
         if defaultExt is not None: return prefix + defaultExt
@@ -132,8 +164,7 @@ def addExt(prefix,
     # already ends with a supported
     # extension, check to see that it
     # exists.
-    if len(allowedExts) == 0 or \
-       any([prefix.endswith(ext) for ext in allowedExts]):
+    if len(allowedExts) == 0 or hasExt(prefix, allowedExts):
         allPaths = [prefix]
 
     # Otherwise, make a bunch of file names, one per
@@ -151,8 +182,14 @@ def addExt(prefix,
         raise PathError('Could not find a supported file '
                         'with prefix "{}"'.format(prefix))
 
-    # Ambiguity! More than one supported
-    # file with the specified prefix.
+    # If ambiguity is ok, return
+    # all matching paths
+    elif not unambiguous:
+        return allPaths
+
+    # Ambiguity is not ok! More than
+    # one supported file with the
+    # specified prefix.
     elif nexists > 1:
 
         # Remove non-existent paths from the
@@ -181,37 +218,61 @@ def addExt(prefix,
     return allPaths[0]
 
 
-def removeExt(filename, allowedExts=None):
+def removeExt(filename, allowedExts=None, firstDot=False):
     """Returns the base name of the given file name.  See :func:`splitExt`. """
 
-    return splitExt(filename, allowedExts)[0]
+    return splitExt(filename, allowedExts, firstDot)[0]
 
 
-def getExt(filename, allowedExts=None):
+def getExt(filename, allowedExts=None, firstDot=False):
     """Returns the extension of the given file name.  See :func:`splitExt`. """
 
-    return splitExt(filename, allowedExts)[1]
+    return splitExt(filename, allowedExts, firstDot)[1]
 
 
-def splitExt(filename, allowedExts=None):
+def splitExt(filename, allowedExts=None, firstDot=False):
     """Returns the base name and the extension from the given file name.
 
-    If ``allowedExts`` is ``None``, this function is equivalent to using::
+    If ``allowedExts`` is ``None`` and ``firstDot`` is ``False``, this
+    function is equivalent to using::
 
         os.path.splitext(filename)
 
-    If ``allowedExts`` is provided, but the file does not end with an allowed
-    extension, a tuple containing ``(filename, '')`` is returned.
+    If ``allowedExts`` is ``None`` and ``firstDot`` is ``True``, the file
+    name is split on the first period that is found, rather than the last
+    period. For example::
+
+        splitExt('image.nii.gz')                # -> ('image.nii', '.gz')
+        splitExt('image.nii.gz', firstDot=True) # -> ('image', '.nii.gz')
+
+    If ``allowedExts`` is provided, ``firstDot`` is ignored. In this case, if
+    the file does not end with an allowed extension, a tuple containing
+    ``(filename, '')`` is returned.
 
     :arg filename:    The file name to split.
 
     :arg allowedExts: Allowed/recognised file extensions.
+
+    :arg firstDot:    Split the file name on the first period, rather than the
+                      last period. Ignored if ``allowedExts`` is specified.
     """
 
-    # If allowedExts is not specified,
-    # we just use op.splitext
+    # If allowedExts is not specified
+    # we split on a period character
     if allowedExts is None:
-        return op.splitext(filename)
+
+        # split on last period - equivalent
+        # to op.splitext
+        if not firstDot:
+            return op.splitext(filename)
+
+        # split on first period
+        else:
+            idx = filename.find('.')
+            if idx == -1:
+                return filename, ''
+            else:
+                return filename[:idx], filename[idx:]
 
     # Otherwise, try and find a suffix match
     extMatches = [filename.endswith(ext) for ext in allowedExts]
@@ -253,7 +314,7 @@ def getFileGroup(path,
         [('.img',    '.hdr'),
          ('.img.gz', '.hdr.gz')]
 
-    If you specify``fileGroups=[('.img', '.hdr')]`` and ``prefix='file'``, and
+    If you specify ``fileGroups=[('.img', '.hdr')]`` and ``prefix='file'``, and
     both ``file.img`` and ``file.hdr`` exist, the :func:`addExt` function would
     return ``file.img`` (i.e. the file which matches the first extension in
     the group).
@@ -281,7 +342,7 @@ def getFileGroup(path,
                       extensions in the group are returned.
 
     :arg unambiguous: Defaults to ``False``. If ``True``, and the path
-                      is not unambiguouosly part of one group, or part of
+                      is not unambiguously part of one group, or part of
                       no groups, a :exc:`PathError` is raised.
                       Otherwise, the path is returned.
     """
@@ -306,12 +367,13 @@ def getFileGroup(path,
         groupFiles = [base + s for s in group]
         exist      = [op.exists(f) for f in groupFiles]
 
-        if any(exist): partialMatches += 1
-        if all(exist): fullMatches    += 1
-        else:          continue
+        if any(exist):
+            partialMatches += 1
 
-        matchedGroups    .append(group)
-        matchedGroupFiles.append(groupFiles)
+        if all(exist):
+            fullMatches += 1
+            matchedGroups    .append(group)
+            matchedGroupFiles.append(groupFiles)
 
     # Path is not part of any group
     if partialMatches == 0:
@@ -398,11 +460,7 @@ def removeDuplicates(paths, allowedExts=None, fileGroups=None):
 
         groupFiles = getFileGroup(path, allowedExts, fileGroups)
 
-        if len(groupFiles) == 0:
-            if path not in unique:
-                unique.append(path)
-
-        elif not any([p in unique for p in groupFiles]):
+        if not any([p in unique for p in groupFiles]):
             unique.append(groupFiles[0])
 
     return unique
@@ -412,7 +470,7 @@ def uniquePrefix(path):
     """Return the longest prefix for the given file name which unambiguously
     identifies it, relative to the other files in the same directory.
 
-    Raises a :exc:`ValueError` if a unique prefix could not be found (which
+    Raises a :exc:`PathError` if a unique prefix could not be found (which
     will never happen if the path is valid).
     """
 
@@ -430,7 +488,7 @@ def uniquePrefix(path):
 
         # Should never happen if path is valid
         elif len(hits) == 0 or idx >= len(filename) - 1:
-            raise ValueError('No unique prefix for {}'.format(filename))
+            raise PathError('No unique prefix for {}'.format(filename))
 
         # Not unique - continue looping
         else:
@@ -439,3 +497,30 @@ def uniquePrefix(path):
             hits    = [h for h in hits if h.startswith(prefix)]
 
     return prefix
+
+
+def commonBase(paths):
+    """Identifies the deepest common base directory shared by all files
+    in ``paths``.
+
+    Raises a :exc:`PathError` if the paths have no common base. This will
+    never happen for absolute paths (as the base will be e.g. ``'/'``).
+    """
+
+    depths = [len(p.split(op.sep)) for p in paths]
+    base   = max(zip(depths, paths), key=operator.itemgetter(0))[1]
+    last   = base
+
+    while True:
+
+        base = op.split(base)[0]
+
+        if base == last or len(base) == 0:
+            break
+
+        last = base
+
+        if all([p.startswith(base) for p in paths]):
+            return base
+
+    raise PathError('No common base')
