@@ -8,14 +8,6 @@
 of information about the current platform we are running on. A single
 ``Platform`` instance is created when this module is first imported, and
 is available as a module attribute called :attr:`platform`.
-
-This module is also home to the following utility functions which abstract
-away various platform differences:
-
-.. autosummary::
-   :nosignatures:
-
-   isWidgetAlive
 """
 
 
@@ -80,32 +72,6 @@ are running the Linux/GTK wx build.
 """
 
 
-def isWidgetAlive(widget):
-    """Returns ``True`` if the given ``wx.Window`` object is "alive" (i.e.
-    has not been destroyed), ``False`` otherwise. Works in both wxPython
-    and wxPython/Phoenix.
-
-    .. warning:: Don't try to test whether a ``wx.MenuItem`` has been
-                 destroyed, as it will probably result in segmentation
-                 faults. Check the parent ``wx.Menu`` instead.
-    """
-
-    import wx
-
-
-    if platform.wxFlavour == platform.WX_PHOENIX:
-        excType = RuntimeError
-    elif platform.wxFlavour == platform.WX_PYTHON:
-        excType = wx.PyDeadObjectError
-
-    try:
-        widget.GetParent()
-        return True
-
-    except excType:
-        return False
-
-
 class Platform(notifier.Notifier):
     """The ``Platform`` class contains a handful of properties which contain
     information about the platform we are running on.
@@ -120,6 +86,7 @@ class Platform(notifier.Notifier):
        os
        frozen
        fsldir
+       fsldevdir
        haveGui
        canHaveGui
        inSSHSession
@@ -142,7 +109,6 @@ class Platform(notifier.Notifier):
         self.WX_MAC_COCOA  = WX_MAC_COCOA
         self.WX_MAC_CARBON = WX_MAC_CARBON
         self.WX_GTK        = WX_GTK
-        self.isWidgetAlive = isWidgetAlive
 
         self.__inSSHSession = False
         self.__inVNCSession = False
@@ -150,7 +116,9 @@ class Platform(notifier.Notifier):
         self.__glRenderer   = None
         self.__glIsSoftware = None
         self.__fslVersion   = None
-        self.fsldir         = os.environ.get('FSLDIR', None)
+
+        # initialise fsldir - see fsldir.setter
+        self.fsldir = self.fsldir
 
         # Determine if a display is available. We do
         # this once at init (instead of on-demand in
@@ -191,13 +159,42 @@ class Platform(notifier.Notifier):
 
     @property
     def haveGui(self):
-        """``True`` if we are running with a GUI, ``False`` otherwise. """
+        """``True`` if we are running with a GUI, ``False`` otherwise.
+
+        This currently equates to testing whether a display is available
+        (see :meth:`canHaveGui`) and whether a ``wx.App`` exists. It
+        previously also tested whether an event loop was running, but this
+        is not compatible with execution from IPython/Jupyter notebook, where
+        the event loop is called periodically, and so is not always running.
+        """
         try:
             import wx
             app = wx.GetApp()
+
+            # TODO Previously this conditional
+            #      also used app.IsMainLoopRunning()
+            #      to check that the wx main loop
+            #      was running. But this doesn't
+            #      suit situations where a non-main
+            #      event loop is running, or where
+            #      the mainloop is periodically
+            #      started and stopped (e.g. when
+            #      the event loop is being run by
+            #      IPython).
+            #
+            #      In c++ wx, there is the
+            #      wx.App.UsesEventLoop method, but
+            #      this is not presently exposed to
+            #      Python code (and wouldn't help
+            #      to detect the loop start/stop
+            #      scenario).
+            #
+            #      So this constraint has been
+            #      (hopefully) temporarily relaxed
+            #      until I can think of a better
+            #      solution.
             return (self.canHaveGui and
-                    app is not None and
-                    app.IsMainLoopRunning())
+                    app is not None)
 
         except ImportError:
             return False
@@ -236,25 +233,23 @@ class Platform(notifier.Notifier):
         :data:`WX_MAC_CARBON`, or :data:`WX_GTK`, indicating the wx platform.
         """
 
-        if not self.haveGui:
+        if not self.canHaveGui:
             return WX_UNKNOWN
 
         import wx
 
         pi = [t.lower() for t in wx.PlatformInfo]
 
-        for tag in pi:
+        if   any(['cocoa'  in p for p in pi]): plat = WX_MAC_COCOA
+        elif any(['carbon' in p for p in pi]): plat = WX_MAC_CARBON
+        elif any(['gtk'    in p for p in pi]): plat = WX_GTK
+        else:                                  plat = WX_UNKNOWN
 
-            if   any(['cocoa'  in p for p in pi]): platform = WX_MAC_COCOA
-            elif any(['carbon' in p for p in pi]): platform = WX_MAC_CARBON
-            elif any(['gtk'    in p for p in pi]): platform = WX_GTK
-            else:                                  platform = WX_UNKNOWN
+        if plat is WX_UNKNOWN:
+            log.warning('Could not determine wx platform from '
+                        'information: {}'.format(pi))
 
-            if platform is WX_UNKNOWN:
-                log.warning('Could not determine wx platform from '
-                            'information: {}'.format(pi))
-
-        return platform
+        return plat
 
 
     @property
@@ -263,7 +258,7 @@ class Platform(notifier.Notifier):
         indicating the wx flavour.
         """
 
-        if not self.haveGui:
+        if not self.canHaveGui:
             return WX_UNKNOWN
 
         import wx
@@ -288,7 +283,13 @@ class Platform(notifier.Notifier):
                   any registered listeners are notified via the
                   :class:`.Notifier` interface.
         """
-        return self.__fsldir
+        return os.environ.get('FSLDIR', None)
+
+
+    @property
+    def fsldevdir(self):
+        """The FSL development directory location. """
+        return os.environ.get('FSLDEVDIR', None)
 
 
     @fsldir.setter
@@ -305,9 +306,9 @@ class Platform(notifier.Notifier):
         elif not op.exists(value): value = None
         elif not op.isdir(value):  value = None
 
-        self.__fsldir = value
-
-        if value is not None:
+        if value is None:
+            os.environ.pop('FSLDIR', None)
+        else:
             os.environ['FSLDIR'] = value
 
             # Set the FSL version field if we can
@@ -315,11 +316,34 @@ class Platform(notifier.Notifier):
 
             if op.exists(versionFile):
                 with open(versionFile, 'rt') as f:
-                    self.__fslVersion = f.read().strip()
+                    # split string at colon for new hash style versions
+                    # first object in list is the non-hashed version string (e.g. 6.0.2)
+                    # if no ":hash:" then standard FSL version string is still returned
+                    self.__fslVersion = f.read().strip().split(":")[0]
 
         self.notify(value=value)
 
 
+    @fsldevdir.setter
+    def fsldevdir(self, value):
+        """Changes the value of the :attr:`fsldevdir` property, and notifies
+        any registered listeners.
+        """
+
+        if value is not None:
+            value = value.strip()
+
+        if   value is None:        pass
+        elif value == '':          value = None
+        elif not op.exists(value): value = None
+        elif not op.isdir(value):  value = None
+
+        if value is None:
+            os.environ.pop('FSLDEVDIR', None)
+        else:
+            os.environ['FSLDEVDIR'] = value
+
+
     @property
     def fslVersion(self):
         """Returns the FSL version as a string, e.g. ``'5.0.9'``. Returns
