@@ -31,8 +31,14 @@ is compatible with PEP 440 (https://www.python.org/dev/peps/pep-0440/):
     which primarily involve bug-fixes and minor changes.
 
 
-The sole exception to the above convention are development versions, which end
-in ``'.dev'``.
+The sole exceptions to the above convention are:
+
+  - development versions, where the point release number is followed by a
+    development release identifier of the form ``'.devN'``, where ``N``
+    denotes a specific development release.
+
+  - Builds, where the version number ends in ``'+buildN'``, where ``N``
+    denotes a specific build.
 """
 
 
@@ -41,7 +47,7 @@ import            re
 import            string
 
 
-__version__ = '1.2.2'
+__version__ = '3.0.1'
 """Current version number, as a string. """
 
 
@@ -53,14 +59,16 @@ def parseVersionString(versionString):
     An error is raised if the ``versionString`` is invalid.
     """
 
-    components = versionString.split('.')
+    # Ignore build if present
+    versionString = versionString.split('+')[0]
+    components    = versionString.split('.')
 
     # Truncate after three elements -
     # a development (unreleased version
     # number will end with '.dev', but
     # we ignore this for the purposes of
     # comparison.
-    if len(components) == 4 and components[3] == 'dev':
+    if len(components) == 4 and components[3].startswith('dev'):
         components = components[:3]
 
     # Major, minor, and point
@@ -80,6 +88,8 @@ def parseVersionString(versionString):
 def compareVersions(v1, v2, ignorePoint=False):
     """Compares the given ``fslpy`` version numbers.
 
+    Both developemnt versions and build numbers are ignored in the comparison.
+
     :arg v1:          Version number to compare
     :arg v2:          Version number to compare
     :arg ignorePoint: Defaults to ``False``. If ``True``, the point release
@@ -120,7 +130,7 @@ def patchVersion(filename, newversion):
     with open(filename, 'rt') as f:
         lines = f.readlines()
 
-    pattern = re.compile('^__version__ *= *\'.*\' *$')
+    pattern = re.compile(r'^__version__ *= *\'.*\' *$')
 
     for i, line in enumerate(lines):
         if pattern.match(line):
